SystemOrganization addCategory: #'OmniST-Benchmarks-Languages-LRSTM'!

BenchmarkGameSuite subclass: #LRSTMBenchmarkGameSuite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Benchmarks-Languages-LRSTM'!

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:16'!
benchAtomicBinaryTrees
	[self benchBinaryTrees ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:17'!
benchAtomicChameleons
	[self benchChameleons ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:17'!
benchAtomicChameneosRedux
	[self benchChameneosRedux ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:17'!
benchAtomicFannkuchRedux
	[self benchFannkuchRedux ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:18'!
benchAtomicFasta
	[self benchFasta ] atomic.! !

!LRSTMBenchmarkGameSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 12/4/2011 12:18'!
benchAtomicNBody
	[self benchNBody ] atomic.! !

Object subclass: #LRSTMBenchmark
	instanceVariableNames: ''
	classVariableNames: 'Value'
	poolDictionaries: ''
	category: 'OmniST-Benchmarks-Languages-LRSTM'!

LRSTMBenchmark class
	instanceVariableNames: 'value mutex doneCount'!

LRSTMBenchmark class
	instanceVariableNames: 'value mutex doneCount'!

!LRSTMBenchmark class methodsFor: 'utilities' stamp: 'lr 5/30/2007 16:10'!
benchmark: aOriginalBlock reference: aReferenceBlock label: aString
	"Assert that aFastBlock is executed faster than aSlow block. The two blocks should be relatively fast and have no side effects, as they are executed multiple times."
	
	| originalTime referenceTime |
	originalTime := aOriginalBlock timeToRun.
	referenceTime := aReferenceBlock timeToRun.
	Transcript 
		show: aString; tab;
		show: ((referenceTime / 1000.0) roundTo: 0.01); tab;
		show: ((originalTime / 1000.0) roundTo: 0.01); tab;
		show: ((originalTime / referenceTime) asFloat roundTo: 0.01); cr! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:16'!
benchmarkActivation
	"self benchmarkActivation"
	
	self 
		benchmark: [ 10000000 timesRepeat: [ [ ] atomic ] ]
		reference: [ 10000000 timesRepeat: [ [ ] value ] ]
		label: 'Activation'! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:17'!
benchmarkBinding
	"self benchmarkBinding"

	self
		benchmark: [ [ 10000000 timesRepeat: [ Value ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ Value ] ] value ]
		label: 'Binding Read'.
	self
		benchmark: [ [ 10000000 timesRepeat: [ Value := nil ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ Value := nil ] ] value ]
		label: 'Binding Write'.! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:17'!
benchmarkInstance
	"self benchmarkInstance"
	
	self 
		benchmark: [ [ 10000000 timesRepeat: [ value ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ value ] ] value ]
		label: 'Instance Read'.
	self
		benchmark: [ [ 10000000 timesRepeat: [ value := nil ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ value := nil ] ] value ]
		label: 'Instance Write'! !

!LRSTMBenchmark class methodsFor: 'btree' stamp: 'StefanMarr 1/4/2012 15:39'!
benchmarkParallel
	"self benchmarkParallel"

	| semaphore data doneCnt repeatCount time btreeCls |
	semaphore := Semaphore forMutualExclusion.
	btreeCls := Smalltalk at: #BTree.
	
	1 to: 70 do: [ :processCount |
		Smalltalk garbageCollect.
		
		data := btreeCls new.
		doneCnt := 0.
		repeatCount := 500.
	
		processCount timesRepeat: [
			[ repeatCount timesRepeat: [ self twoAtomicUpdates: data ].
			  semaphore critical: [ doneCnt := doneCnt + 1 ] ]
				forkAt: Processor activeProcess priority ].
		time := [
			[ doneCnt < processCount ]
				whileTrue: [ Processor yield ] ]
			timeToRun.

		Transcript show: processCount; tab; show: time; cr ]! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:17'!
benchmarkSend
	"self benchmarkSend"
	
	self
		benchmark: [ [ 10000000 timesRepeat: [ nil flag: #zork ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ nil flag: #zork ] ] value ]
		label: 'Message invokation'.
	self
		benchmark: [ [ 10000000 timesRepeat: [ nil flag: #zork ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ 1 + 2 ] ] value ]
		label: 'Message invokation (fast)'! !

!LRSTMBenchmark class methodsFor: 'benchmarks' stamp: 'lr 5/30/2007 16:18'!
benchmarkVariable
	"self benchmarkVariable"

	self 
		benchmark: [ [ 10000000 timesRepeat: [ #( nil ) at: 1 ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ #( nil ) at: 1 ] ] value ]
		label: 'Variable Read'.
	self
		benchmark: [ [ 10000000 timesRepeat: [ #( nil ) at: 1 put: nil ] ] atomic. [] value ]
		reference: [ [] atomic. [ 10000000 timesRepeat: [ #( nil ) at: 1 put: nil ] ] value ]
		label: 'Variable Write'! !

!LRSTMBenchmark class methodsFor: 'btree' stamp: 'lr 5/31/2007 17:11'!
twoAtomicUpdates: aDictionary
	[ [ aDictionary 
		at: SmallInteger maxVal atRandom put: 0;
		at: SmallInteger maxVal atRandom put: 0 ]
			atomic ]
		on: LRSTMConflict
		do: [ self twoAtomicUpdates: aDictionary ]! !

!LRSTMBenchmark class methodsFor: 'btree' stamp: 'lr 6/1/2007 11:01'!
twoMutexUpdates: aDictionary
	mutex critical: [
		aDictionary
			at: SmallInteger maxVal atRandom put: 0;
			at: SmallInteger maxVal atRandom put: 0 ]! !

SMarkLoops subclass: #LRSTMBenchLoops
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Benchmarks-Languages-LRSTM'!

!LRSTMBenchLoops methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/4/2011 12:04'!
benchAtomicFloatLoop
	[ self benchFloatLoop ] atomic.! !

!LRSTMBenchLoops methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/4/2011 12:04'!
benchAtomicIntLoop
	[ self benchIntLoop ] atomic.! !
